import React, { useMemo, useState } from 'react';
import { Table } from 'json-server-orm';
import { log } from './common/log';
import { env } from './common/env';
import { TableUI } from './Table';
import { ObjectUI } from './Object';
import './Client.scss';
export function Client({ db, validation }) {
    if (!env.db)
        env.db = db;
    if (!env.validation)
        env.validation = validation;
    const names = useMemo(() => Object.keys(db), []);
    const rawname = useMemo(() => new URL(location.href).searchParams.get('table') ?? names[0], []);
    const [name, setName] = useState(rawname);
    const isTable = useMemo(() => db[name] instanceof Table, [name]);
    const [limit, setLimit] = useState(10);
    const [q, setQ] = useState('');
    log('[App] begin', name, isTable ? 'table' : 'object');
    return React.createElement(React.Fragment, null,
        React.createElement("div", { className: 'panel' },
            React.createElement("select", { className: 'names', value: name, onChange: (ev) => setName(ev.target.value) }, names.map(o => React.createElement("option", { key: o, value: o }, o))),
            isTable && React.createElement(React.Fragment, null,
                React.createElement("select", { className: 'limit', value: limit, onChange: (ev) => (setLimit(parseInt(ev.target.value))) }, [1, 10, 50, 100, 0].map(o => React.createElement("option", { key: o, value: o }, o === 0 ? 'All' : (o + ' items')))),
                React.createElement("input", { className: 'q', type: 'text', placeholder: 'search', value: q, onChange: (ev) => setQ(ev.target.value) }))),
        isTable
            ? React.createElement(TableUI, { db: db, name: name, limit: limit, q: q })
            : React.createElement(ObjectUI, { db: db, name: name }));
}
