import React, { useEffect, useMemo, useState } from 'react';
import { cloneDeep } from 'lodash';
import { ValidationError } from 'json-server-orm';
import { log } from './common/log';
export function ObjectUI({ name, db }) {
    const dbObject = useMemo(() => db[name], [name]);
    const [object, setObject] = useState();
    const [editable, setEditable] = useState(false);
    useEffect(() => {
        (async () => {
            try {
                const obj = await dbObject.one();
                log('[Object]', obj);
                setObject(obj);
                setEditable(false);
            }
            catch (error) {
                alert(error.message);
            }
        })();
    }, [name]);
    const handleUpdate = async (obj) => {
        setObject(obj);
        setEditable(false);
    };
    return React.createElement("div", { className: 'object' }, editable
        ? React.createElement(EditableRow, { db: db, name: name, object: object, onCancel: () => setEditable(false), onSubmit: handleUpdate })
        : React.createElement(Row, { object: object, onEdit: () => setEditable(true) }));
}
function EditableRow({ db, name, object, onSubmit, onCancel }) {
    const dbObject = useMemo(() => db[name], [name]);
    const [obj, setObj] = useState(cloneDeep(object));
    const [error, setError] = useState();
    const handleUpdate = async () => {
        try {
            const o = await dbObject.update(obj, true);
            console.log(o);
            onSubmit && onSubmit(o);
        }
        catch (err) {
            if (err.response)
                err = new ValidationError(err.response.data.message, err.response.data.field);
            if (err instanceof ValidationError && err.field)
                setError(err);
            else
                alert(err.message);
        }
    };
    return React.createElement(React.Fragment, null,
        React.createElement("textarea", { className: 'type-object', value: typeof obj === 'string' ? obj : JSON.stringify(obj, null, 2), onChange: (ev) => setObj(ev.target.value), onBlur: (ev) => setObj(JSON.parse(ev.target.value)) }),
        error && React.createElement("div", { className: 'error' },
            "[",
            error.field,
            "]: ",
            error.message),
        React.createElement("br", null),
        React.createElement("button", { className: 'icon submit', onClick: handleUpdate }, "Submit"),
        React.createElement("button", { className: 'icon cancel', onClick: () => onCancel && onCancel() }, "Cancel"));
}
function Row({ object, onEdit }) {
    return React.createElement(React.Fragment, null,
        React.createElement("pre", { className: 'type-object', onClick: () => onEdit && onEdit() }, JSON.stringify(object, null, 2)),
        React.createElement("button", { className: 'icon edit', onClick: () => onEdit && onEdit() }, "Edit"));
}
