import React, { useEffect, useMemo, useState } from 'react';
import useForceUpdate from 'use-force-update';
import { log } from './common/log';
import { Row } from './Row';
import { EditableRow } from './EditableRow';
import { env } from './common/env';
export function TableUI({ name, limit, q, db }) {
    const validation = env.validation;
    const forceUpdate = useForceUpdate();
    const fields = useMemo(() => validation.getOwnFields(name), [name]);
    const dbTable = useMemo(() => db[name], [name]);
    const [items, setItems] = useState([]);
    const [addable, setAddable] = useState(false);
    const [editId, setEditId] = useState(0);
    const [count, setCount] = useState(0);
    const [sort, setSort] = useState(['id', 'desc']);
    useEffect(() => {
        (async () => {
            try {
                const opts = { sort: sort[0], order: sort[1] };
                if (limit)
                    opts.limit = limit;
                if (q)
                    opts.q = q;
                const its = await dbTable.all(opts);
                log(`[Table] ${its.length} items loaded, opts`, opts);
                setItems(its);
                setCount(await dbTable.count());
            }
            catch (error) {
                alert(error.message);
            }
        })();
    }, [name, limit, q, sort]);
    const handleDelete = async (id) => {
        items.splice(items.findIndex(o => o.id === id), 1);
        forceUpdate();
        setCount(await dbTable.count());
    };
    const handleUpdate = async (item) => {
        Object.assign(items.find(o => o.id === item.id), item);
        setEditId(0);
    };
    const handleAdd = async (item) => {
        items.unshift(item);
        setAddable(false);
        setCount(await dbTable.count());
    };
    const handleSort = async (field) => {
        setSort([field, sort[1] === 'desc' ? 'asc' : 'desc']);
    };
    return !fields ? null : React.createElement("table", { className: 'table table-light' },
        React.createElement("thead", null,
            React.createElement("tr", null,
                React.createElement("th", null,
                    !addable &&
                        React.createElement("i", { className: 'icon add', onClick: () => setAddable(true) }),
                    count),
                fields.map(f => (React.createElement("th", { key: f, className: f === sort[0] ? sort[1] : '', onClick: () => handleSort(f) }, f))))),
        React.createElement("tbody", null,
            addable &&
                React.createElement(EditableRow, { table: name, onSubmit: handleAdd, onCancel: () => setAddable(false) }),
            items.map((item) => editId !== item.id
                ? React.createElement(Row, { key: item.id + '', name: name, item: item, onDelete: handleDelete, onEdit: () => setEditId(item.id) })
                : React.createElement(EditableRow, { key: item.id + '', table: name, item: item, onSubmit: handleUpdate, onCancel: () => setEditId(0) }))));
}
